<?php

require_once('api.actions.php');
require_once('api.events.php');
require_once('api.navigation.php');

/**
* @desc nx_fire_event returns an array of results returned
* from each event handler.  Use this function to quickly
* search the results for a particular value.  Useful when 
* searchign for boolean false/true.
* @param array $results An array of result values
* @param mixed $mixed The value to compare against.
* @return bool Returns true if result found, false otherwise.
*/
function nx_find_in_results($results,$mixed)
{
    if (is_array($results))
    {
        foreach($results as $val)
        {            
            if ($mixed == $val)
            {
                return true;
            }
        }
    }
        
    return false;
}
    
/**
 * Register a new delegate (or hook) for an event
 * @param string $eventType The type of handler to hook into
 * @param string $handlerFunction The function to be called when the 
 *			event occurs
 */
function nx_register_handler($eventType, $handlerFunction)
{
	global $HANDLERS, $EVENT_TYPES;
	
	if (in_array($eventType,$EVENT_TYPES))
	{
		if (!isset($HANDLERS[$eventType]))
		{
			$HANDLERS[$eventType] = array();
		}
		
		$HANDLERS[$eventType][] = array ('function'=>$handlerFunction);
	}
}

/**
 * Called by nexus engine when an event occurs.  All registered handlers
 *	will be invoked
 * @param string $eventType The type of event being registered
 * @param array $params An associative array of paramaters associated
 * 			with this event.
 * @return array Returns an array of results from the handlers.
 */
function nx_fire_event($eventType,$params)
{
	global $HANDLERS, $EVENT_TYPES;

    $results = array();
	if (isset ($HANDLERS[$eventType]))
	{
		foreach($HANDLERS[$eventType] as $handler)
		{
			$results[] = $handler['function']($params);
		}
	}
    
    return $results;
}

/**
* @desc Retrieves all the actions that an administrator 
*       can perform as an associative array
* @return array An array with the following format:
*           {
*               name=>string, 
*               link=>string, 
*               icon=string,
*               parent[optional]=boolean,
*               children[optional]={action array}
*           }
*/
function nx_get_user_actions()
{
    $actions = array();
    $user = app()->getLoggedInUserObject();
    $permGroup = $user->getPermissions()->getPermissionGroup();
    
    // Browsing Actions
    $browseActions = new ActionItem(str('STR_BROWSE'),'','');
    $browseActions->parent = true;
    $browseActions->children[] = new ActionItem( 
                                str('STR_ACTION_BROWSELIB'),
                                'index.php?'.URLARG_ACTION.'='.URLACTION_BROWSELIBRARY,
                                smarty_templ_file_path('images/library.jpg'));
                                
    $browseActions->children[] = new ActionItem( 
                                str('STR_ACTION_BROWSEBYCAT'),
                                'index.php?'.URLARG_ACTION.'='.URLACTION_BROWSEBYCAT,
                                smarty_templ_file_path('images/browse_bycat.jpg'));
                                                                                 

	if ($permGroup == USER_TYPE_ADMIN)
	{
		$browseActions->children[] = new ActionItem( 
	                                str('STR_ACTION_BROWSEBYORG'),
	                                'index.php?'.URLARG_ACTION.'='.URLACTION_BROWSEBYORG,
	                                smarty_templ_file_path('images/browse_byorg.jpg'));
	    
	}

    $actions[] = $browseActions;                                                                

    if ($permGroup == USER_TYPE_ADMIN || $permGroup == USER_TYPE_DOMAIN_ADMIN || $permGroup == USER_TYPE_DOMAIN_GROUP_ADMIN)
    {
        // Add Actions
        $addActions = new ActionItem(str('STR_ADD'),'','');
        $addActions->parent = true;
        
        if ($permGroup == USER_TYPE_ADMIN)
        {
            $addActions->children[] = new ActionItem( 
                                        str('STR_ACTION_ADDGROUP'),
                                        'index.php?'.URLARG_ACTION.'='.URLACTION_MODGROUP,
                                        smarty_templ_file_path('images/new_group.jpg'));
        }

        if ($permGroup == USER_TYPE_DOMAIN_ADMIN)
        {
            $addActions->children[] = new ActionItem( 
                                        str('STR_ACTION_EDITYOURORG'),
                                        'index.php?'.URLARG_ACTION.'='.URLACTION_MODCOMPANY.'&'.URLARG_ID.'='.$user->getCompany()->get(PROP_ID),
                                        smarty_templ_file_path('images/new_org.jpg'));
        }
        
        if ($permGroup == USER_TYPE_ADMIN)
        {
            $addActions->children[] = new ActionItem( 
                                        str('STR_ACTION_ADDORG'),
                                        'index.php?'.URLARG_ACTION.'='.URLACTION_MODCOMPANY,
                                        smarty_templ_file_path('images/new_org.jpg'));
        }
        
        $addActions->children[] = new ActionItem( 
                                    str('STR_ACTION_ADDUSER'),
                                    'index.php?'.URLARG_ACTION.'='.URLACTION_MODUSER,
                                    smarty_templ_file_path('images/new_user.jpg'));

        if ($permGroup == USER_TYPE_ADMIN)
        {
            $addActions->children[] = new ActionItem( 
                str('STR_ACTION_ADDCAT'),
                'index.php?'.URLARG_ACTION.'='.URLACTION_MODCAT,
                smarty_templ_file_path('images/new_cat.jpg'));
        }

        $addActions->children[] = new ActionItem( 
                                    str('STR_ACTION_ADDDOC'),
                                    'index.php?'.URLARG_ACTION.'='.URLACTION_MODDOC,
                                    smarty_templ_file_path('images/doc_icon_add_large.jpg'));
        
                               
        $actions[] = $addActions;                                    
        
        //  Other Actions
        $otherActions = new ActionItem(str('STR_OTHER_ACTIONS'),'','');
        $otherActions->parent = true;
        
        if ($permGroup == USER_TYPE_ADMIN)
        {
            $otherActions->children[] = new ActionItem(
                                str('STR_ACTION_MANAGELIB'),
                                'index.php?'.URLARG_ACTION.'='.URLACTION_DOCMANAGER,
                                smarty_templ_file_path('images/manage_library.jpg'));

            $otherActions->children[] = new ActionItem(
                                str('STR_ACTION_MANAGEHISTORY'),
                                'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWUSAGE,
                                smarty_templ_file_path('images/reports.jpg'));
                                
            $otherActions->children[] = new ActionItem(
                            str('STR_ACTION_MANAGEUSERS'),
                            'index.php?'.URLARG_ACTION.'='.URLACTION_USERMANAGER,
                            smarty_templ_file_path('images/manage_users.jpg'));
        }
        
        $otherActions->children[] = new ActionItem(
                            str('STR_ACTION_SENDMAILTOGROUP'),
                            'index.php?'.URLARG_ACTION.'='.URLACTION_SENDMAIL_TO_GROUP,
                            smarty_templ_file_path('images/mail_groups.jpg'));

        if ($permGroup == USER_TYPE_ADMIN)
        {
            $otherActions->children[] = new ActionItem(
                                str('STR_ACTION_EDITSETTINGS'),
                                'index.php?'.URLARG_ACTION.'='.URLACTION_SETTINGS,
                                smarty_templ_file_path('images/edit_settings.jpg'));

            $otherActions->children[] = new ActionItem(
                                str('STR_ACTION_UPDATEPLUGINS'),
                                'index.php?'.URLARG_ACTION.'='.URLACTION_PLUGINS,
                                smarty_templ_file_path('images/edit_plugins.png'));
                                
            $otherActions->children[] = new ActionItem(
                                str('STR_ACTION_SERVERINFO'),
                                'index.php?'.URLARG_ACTION.'='.URLACTION_SERVER_INFO,
                                smarty_templ_file_path('images/server_info.jpg'));
        }
                            
        $actions[] = $otherActions;                                                                                                                        
    }
    
    nx_fire_event('EVENT_FRONTPANEL_LINKS_NEEDED',array('actions'=>&$actions));
    
    return $actions;
}

/**
* @desc Called when the navbar (be it a sidebar or some other
*          UI entity) needs to be constructed.  
* @param bool $veritcal Set to true if this is for a vertical nav area or
*           false for a horizontal nav area
* @return array An array of NavBlock objects
*/
function nx_get_nav_blocks($vertical)
{
    $blocks = array();
    
    $user = app()->getLoggedInUserObject();
    if ($user)
    {
        $permissionGroup = $user->getPermissions()->getPermissionGroup();
            
        if ($vertical)
        {
			if ($permissionGroup == USER_TYPE_ADMIN)
			{
	            ///////// Managers
	            $block = new NavBlock();
	            $block->title = str('STR_NAV_MANAGE');
	            $block->showTitle = true;

	            $item = new NavItem;
	            $item->title = str('STR_ACTION_MANAGEUSERS');
	            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_USERMANAGER;
	            $item->icon = smarty_templ_file_path('images/edit_user.jpg');        
	            $block->items[] = $item;				

	            $item = new NavItem;
	            $item->title = str('STR_ACTION_MANAGELIB');
	            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_DOCMANAGER;
	            $item->icon = smarty_templ_file_path('images/manage_library.jpg');        
	            $block->items[] = $item;
	
				$blocks[] = $block;
			}	
			
            ///////// Library Section
            $block = new NavBlock();
            $block->title = str('STR_NAV_LIBRARY');
            $block->showTitle = true;
            
            $item = new NavItem;
            $item->title = str('STR_ACTION_BROWSELIB');
            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_BROWSELIBRARY;
            $item->icon = smarty_templ_file_path('images/library.jpg');        
            $block->items[] = $item;
            
            if ($permissionGroup != USER_TYPE_REGISTERED)
            {
                $item = new NavItem;
                $item->title = str('STR_ACTION_ADDDOC');
                $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_MODDOC;
                $item->icon = smarty_templ_file_path('images/doc_icon_add_large.jpg');        
                $block->items[] = $item;
            }
            
            if ($permissionGroup == USER_TYPE_ADMIN)
            {
                $item = new NavItem;
                $item->title = str('STR_ACTION_ADDMULTIDOC');
                $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_MODDOC.'&amp;'.URL_MULTIUPLOAD.'=1';
                $item->icon = smarty_templ_file_path('images/add_multidoc.jpg');        
                $block->items[] = $item;        
            }
            
            $blocks[] = $block;
            
            ///////// Filebox Section
            $block = new NavBlock();
            $block->title = str('STR_NAV_FILEBOX');
            $block->showTitle = true;
                    
            $item = new NavItem;
            $item->title = str('STR_ACTION_BROWSEFILEBOX');
            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWPOOL;
            $item->icon = smarty_templ_file_path('images/browse_filebox.jpg');        
            $block->items[] = $item;        
            
            $item = new NavItem;                
            $item->title = str('STR_ACTION_BROWSESHAREDFILEBOX');
            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWPOOL.'&amp;shared=1';
            $item->icon = smarty_templ_file_path('images/browse_filebox_shared.jpg');        
            $block->items[] = $item;         
            
            $item = new NavItem;
            $item->title = str('STR_ACTION_ADDFILEBOX');
            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_MOD_POOL;
            $item->icon = smarty_templ_file_path('images/add_filebox.jpg');        
            $block->items[] = $item;        

            $item = new NavItem;
            $item->title = str('STR_ACTION_ADDMULTIDOC');
            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_MOD_POOL.'&'.URL_MULTIUPLOAD.'=1';
            $item->icon = smarty_templ_file_path('images/add_multidoc.jpg');        
            $block->items[] = $item;        
            
            $blocks[] = $block;
            
            ///////// Trash
            if ($permissionGroup != USER_TYPE_REGISTERED)
            {
                $block = new NavBlock();
                $block->title = str('STR_NAV_TRASHCAN');
                $block->showTitle = true;
                
                $item = new NavItem;
                $item->html ="<div id=\"trashcan\"></div><script type=\"text/javascript\">new YAHOO.util.DDTarget(\"trashcan\");</script>";
                $block->items[] = $item;
                
                $item = new NavItem;
                $item->title = str('STR_ACTION_VIEWTRASH');
                $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWTRASH;
                $item->icon = smarty_templ_file_path('images/browse_trash.jpg');
                $block->items[] = $item;        
                
                $blocks[] = $block;
            }

            ///////// Briefcase
            $block = new NavBlock();
            $block->title = str('STR_NAV_BRIEFCASE');
            $block->showTitle = true;
            
            $item = new NavItem;
            $item->html ="<div id=\"briefcase\"></div><script type=\"text/javascript\">new YAHOO.util.DDTarget(\"briefcase\");</script>";
            $block->items[] = $item;
            
            $item = new NavItem;
            $item->title = str('STR_ACTION_VIEWBRIEFCASE');
            $item->link = 'index.php?'.URLARG_ACTION.'='.URLACTION_VIEWBIN;
            $item->icon = smarty_templ_file_path('images/view_briefcase.jpg');
            $block->items[] = $item;        
            
            $blocks[] = $block;
        }  
              
    }
    
    nx_fire_event('EVENT_NAV_NEEDED',array('vertical'=>$vertical,'blocks'=>&$blocks));
    
    return $blocks;
}

/**
 * Returns the username of the currently logged in user. Returns an empty
 * string if there is no user logged in
 * @return string The username
 */
function nx_get_current_username()
{
    return app()->getLoggedInUsername();
}

/**
 * Empties the trash for the currently logged in user
 * @return bool Returns true if successful, false otherwise
 */
function nx_empty_trash()
{
	$trash = new shTrashCollection;
	if ($trash->emptyTrash())
    {
        return true;
    }
    
    return false;
}

/**
 * Returns the groups within the given company as an array of associative
 * arrays where the associatve arrays contain information about each group
 * @param number $company_id The id of the company for which groups are to be
 *              returned
 * @return array An array of group arrays
 */
function nx_get_groups_in_company($company_id)
{
    $groupsAsArrays = array();
	$company = new shCompany;
	if ($company->loadId($company_id))
	{
	    $groups = $company->getGroups();
	    
	    foreach($groups as $group)
	    {
	        $groupsAsArrays[] = $group->toArray();
	    }
			    
	}
    return $groupsAsArrays;
}

/**
 * Adds the given document to the briefcase
 * @param int $document_id The Id of the document to add
 * @return bool Returns true if added, false otherwise
 */
function nx_add_document_to_briefcase($document_id)
{
	if (!shItemBin::isItemInBin($document_id))
	{
		shItemBin::addItemToBin ($document_id);
        return true;
    }
    
    return false;
}

/**
 * Removes a tag from the given document
 * @param int $document_id The Id of the document from which the tag is to be
 *  removed
 * @param string $tag The tag string to remove from the document
 * @return Returns true if removed, false otherwise.
 */
function nx_remove_document_tag($document_id,$tag)
{
    $query = 'select * from sh_attributes where attribute_name="'.$tag.'"';		
    $tagCollection = new shAttributeCollection;
    $tagCollection->loadFromQuery ($query);
    $tag = null;
    if ($tagCollection->count() == 0)
    {
        return false;
    }
    else 
    {
        $tag = $tagCollection->getAt (0);
    }
        
    // First, get the id of the attribute
    $item = new shItemDocument;
    $item->set (PROP_ID, $document_id);
    if ($item->load())
    {			
        if ($item->hasAttribute($tag->get(PROP_NAME)))	
        {
            $item->removeAttribute ($tag);            
            return $item->update();
        }
    }
                                
    return false;
}

/**
 * Adds a list of tags (separated by spaces) to the given document
 * @param int $document_id The id of the document
 * @param string The tags to add (separated by spaces)
 * @return Returns true if added, false otherwise)
 */
function nx_add_tag_list_to_document($document_id, $tags)
{
    $tagList = split(' ',trim($tags));
    
    foreach($tagList as $tagString)
    {
        $actions = new shActions();
        $tagObject = $actions->doAddTag($tagString);    
        $actions->doApplyTagToDocument($document_id,$tagObject);
    }
    
    return (count($tagList) > 0);    
}

/**
 * Changes the parent of a category effectively moving it from its current
 * position in the hierarchy to a new position.
 * @param int $parent_category_id The new parent
 * @param int $child_category_id The id of the category to move
 * @return Returns true if moved, false otherwise.
 */
function nx_set_category_parent($parent_category_id,$child_category_id)
{
    $cat = new shCategory;
    $cat->set (PROP_ID, $child_category_id);
    if ($cat->load ())
    {
        $cat->set(PROP_PARENT,$parent_category_id);
        if ($cat->update())
        {
            return true;
        }
    }
    return false;    
}

/**
* @desc Adds a category to the list of categories for a document.
* @param int $document_id The id of the document
* @param int $category_id The new category
* @param int $remove_from_category_id The category to remove from the list of categories. By
*       specifying this value you are effectively moving the document from one category
*       to another.  Leave as -1 to only add a new category.
* @return bool Returns true if added, false otherwise.
*/
function nx_add_document_category($document_id, $category_id, $remove_from_category_id=-1)
{
    $doc = new shItemDocument;
    $doc->set (PROP_ID, $document_id);
    if ($doc->load ())
    {
        $doc->addCategory ($category_id);
        
        if ($remove_from_category_id > -1)
        {
            $doc->removeCategory ($remove_from_category_id);
        }
        
        if ($doc->update())
        {
            return true;
        }
    }
    return false;    
}

/**
* @desc Moves an object to the trash
*/
function nx_move_to_trash($object_id, $object_type)
{
    if (($object_type != OBJECT_TYPE_UNKNOWN) && 
        ($object_id != 0))
    {       
    
        $item = null;
        switch ($object_type)
        {
            case OBJECT_TYPE_ITEM:
                $item = new shItemDocument;
                break;
            case OBJECT_TYPE_USER:
                $item = new shUser;
                break;
            case OBJECT_TYPE_GROUP:
                $item = new shGroup;
                break;
            case OBJECT_TYPE_COMPANY:
                $item = new shCompany;
                break;
            case OBJECT_TYPE_ATTRIBUTE:
                $item = new shAttribute;
                break;
            case OBJECT_TYPE_CATEGORY:
                $item = new shCategory;
                break;
        }
        
        if ($item)
        {
            $item->set(PROP_ID,$object_id);
            if ($item->delete())
            {
                return true;
            }
        }
        
    }
    
    return false;
}

/**
* @desc Have the current user get email notifications when the category has changed
* @param int $category_id The id of the category to generate notifications
* @param int $change_type The type of action that will generate notifications
*           (NOTIFY_CHANGE_TYPE_UPDATE or NOTIFY_CHANGE_TYPE_ADD)
*/
function nx_register_category_notification($category_id, $change_type)
{
    $notify = new shUserNotification;
    if ($notify->isRegisteredForCategoryNotification($category_id, $change_type))
    {
        return $notify->removeCategoryNotification ($category_id, $change_type, nx_get_current_username());
    }
    else
    {
        return $notify->registerForCategoryNotification ($category_id, $change_type, nx_get_current_username());
    }    
}

/**
* @desc Retrieves a document conversation in the form of an array of associative
* arrays where each top-level array item is a post in the conversation and each
* associative array item is a property of that post.                                       
* @param int $document_id The id of the document for which a conversation is to be retrieved
* @return array Returns an array of associative arrays (see description)
*/
function nx_get_conversation($document_id)
{
    $doc = new shItemDocument();
    $doc->set(PROP_ID, $document_id);
    
    // we don't need to load everything here.  Just get the smarty variables.
    $conversation = $doc->getConversation();
    $conversationAsArray = array();
    foreach($conversation as $post)
    {
        $postAsArray = $post->toArray();
        $postAsArray['datePosted'] = iwUtilities::dateTimeFormat($postAsArray['datePosted'],true);
        $conversationAsArray[] = $postAsArray;
    }
    
    return $conversationAsArray;
}

/**
 * Deletes the post given permanently
 * @param int $post_id The ID of the post to delete
 * @return bool Returns true if deleted successfully false otherwise.
 */
function nx_delete_conversation_post($post_id)
{
	$post = new shPost;
	if ($post->loadFromId($post_id))
	{
		if ($post->delete())
		{
			app()->invalidateTable('sh_item_conversations');
			return true;
		}
	}
	
	return false;
}

/**
 * Adds all the documents in the given category (immediate children only) to
 * the briefcase. 
 * @param int $category_id The id of the category from which documents will be added
 * @return int Returns the number of documents added
 */
function nx_add_documents_in_category_to_briefcase($category_id)
{
    $cat = new shCategory;
    $cat->set(PROP_ID, $category_id);
    if ($cat->load())
    {
        $count = 0;
        $coll = $cat->getItemsInCategoryCollection();
        foreach ($coll->items() as $doc)
        {
            if (nx_add_document_to_briefcase($doc->get(PROP_ID)))
            {
                $count++;
            }
        }
        
        return $count;
    }
       
    return 0;
}

/**
* @desc Returns the number of documents in the briefcase
* @return int The number of documents in the briefcase
*/
function nx_get_number_of_docs_in_briefcase()
{
    return shItemBin::getCount();
}

/**
* @desc Renames a document in the library
* @param int $document_id The id of the document to rename
* @param string $new_name The new name of the document
* @return bool Returns true if renamed, false otherwise;
*/
function nx_rename_document($document_id,$new_name)
{
    $doc = new shItemDocument();
    $doc->set(PROP_ID, $document_id);
    if ($doc->load())
    {
        $doc->set(PROP_NAME,$new_name);
        if ($doc->update())
        {
            return true;
        }
    }
    
    return false;    
}

/**
* @desc Renames a category in the library
* @param int $category_id The id of the category to rename
* @param string $new_name The new name of the category
* @return bool Returns true if renamed, false otherwise;
*/
function nx_rename_category($category_id,$new_name)
{
    $cat = new shCategory();
    $cat->set(PROP_ID, $category_id);
    if ($cat->load())
    {
        $cat->set(PROP_NAME,$new_name);
        if ($cat->update())
        {
            return true;
        }
    }
    
    
    return false;
}

/**
* @desc Retrieves an array of all usernames
* @return array Returns an array of usernames
*/
function nx_get_usernames()
{
	$params = new iwParameterCollection();
	$params->add(new iwParameter(FILTER_STATUS,USER_STATUS_ACTIVE));
	$users = new shUserCollection();
	$usernames = array();
	$users->load($params);
	foreach($users->items() as $user)
	{			
		$usernames[] = $user->get(PROP_USERNAME);			
	}
	return $usernames;
}

/**
* @desc Determines if there is a user logged into the system
* @return bool Returns true if logged in, false otherwise.
*/
function nx_is_user_logged_in()
{
    return (app()->getLoggedInUserObject() instanceof shUser);
}

/**
* @desc Gets a list of documents from the library in the form
*       of an array of associatve arrays
* @return array Returns an array of associatvie arrays
*/
function nx_get_documents()
{
    $coll = new shItemDocumentCollection();

    $collectionParams = new iwParameterCollection();
    
    // Don't list items in trash
    $p = new iwParameterArray(FILTER_STATUS);
    $p->addItem_nokey (ITEM_STATUS_ACTIVE);
    
    $p = $collectionParams->add($p);
    
    $coll->load ($collectionParams);
    
    $final = array();
    foreach ($coll->items() as $item)
    {
        $final[] = $item->toArray();
    }
    
    return $final;
}

/**
 * Share a filebox document with another user
 * @param int $document_id The id of the document to share
 * @param string $username The name of the user to share with
 * @return bool Returns true if shared, false otherwise
 */
function nx_share_filebox_document($document_id,$username)
{
	$doc = new shPoolItem;
	if ($doc->loadId($document_id))
	{
		return $doc->share($username);
	}
	
	return false;
}

/**
 * @desc No longer shares a filebox document with a specific user
 * @param int $document_id The id of the document to unshare
 * @param string $username The name of the user to unshare with
 * @return bool Returns true if unshared, false otherwise
 */
function nx_unshare_filebox_document($document_id,$username)
{
	$doc = new shPoolItem;
	if ($doc->loadId($document_id))
	{
		return $doc->unshare($username);
	}
	
	return false;
   
}

/**
* @desc Returns an array of assocative arrays describing files that 
* were recently uploaded.  Array items look like this:
*   filename => Original filename
*   filesize => File size
*   upload_message => Description of upload problem, if any
*   upload_result => The result of an upload attempt
* @return array An array of arrays that look like what is described above.
*/
function nx_get_recently_uploaded_files()
{
    $results = array();
    $files = uploads()->getRecentlyUploadedFiles();
    foreach($files as $file)
    {
        $results[] = array(
            'filename'=>$file->originalFilename,
            'filesize'=>$file->fileSize,
            'upload_message'=>$file->uploadResultMessage,
            'upload_result'=>$file->uploadResult);
    }
    
    return $results;
}

/**
 * Loads the HTML from a template using the given template variable
 *	dictionary (name=>value pairs) and the relative path to the template
 *	starting from the active theme's root directory
 * @param array $templateVariables An array of name/value pairs used to populate the template
 * @param string $templateRelPath The relative path to the template file (relative to the root of the active theme)
 * @return string Returns the HTML
 */
function nx_load_template($templateVariables,$templateRelPath)
{
	$smarty = app()->getSmartyObject();
	foreach($templateVariables as $key=>$value)
	{
		$smarty->assign($key,$value);
	}
	
	return smarty_templ_fetch($templateRelPath,$smarty);
}

?>