<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**  
 *	Represents an object that has a presence in a database
 *	Derive from this object if your new object represents a record
 *	in the database.  This class provides very useful methods for
 *	accessing the database such as adding,removing and marking records
 *	as deleted.  Certain method MUST be overridden, though
 */
abstract class iwDbTable extends iwObject
{
	/**
	 * When this is set to true, anytime 'delete' is called, the
	 * record is permanently removed from the database rather than
	 * simply being marked as deleted 
	 */
	protected $_trueDeletionEnabled = false;

	/** Constructor: Calls _addsProperties function to initialize object properties
	*/
	function __construct () 
	{
		$this->addProperties ();
	}

	/**  
	 *	A dynamic list of properties that are associated with this object.	
     *	This need only be called once per instantiation of this class
     *	and is handled automatically  by the base class as long
     *	as its constructor is called.
     */
	protected function addProperties ()	{}

	/**
	 * Called when a record in the represented table has been permanently 
	 * delted.  This is useful to remove any references to this record in 
	 * any other tables for those database which don't support referential 
	 * integrity.
	 */	 	 	 	 	
	protected function onAfterPermanentDelete (){}
	
	/**  
	 *	Gets all the database field names with associated values.
     *	This will return an associative array where the keys
     *	are the table field names and the values are the values
     *	stored in this class.
     *	@param bool $includeDBKey If true, then the key field for the table is returned in the array as well.
     *	@return array An associative array of dbase fields => class values
     *	@access public
	 */
	abstract public function getDBKeyValueArray ($includeDBKey);

	/**  
	 *	Sets all the object properties based on the given database field values.	
     *	Given is an associative array where the keys are the database field names
     *	and the values are the values of those fields.
     *	@param array $dbFieldsAndValues The associative array of database 
     *		 field names and values.
     *	@return bool True if all the required fields were found and copied over.
     *	@access protected
	 */
	abstract public function setDBKeyValueArray ($dbFieldsAndValues);

	/**  
	 *	Gets the associative array required to mark a record as deleted
	 * 	@param bool $isRestoring If this is true then value to be returned should
	 *					be the restoration information.
     *	@return array An associative (key>value) array populated with the 
     *		 keys/values required to mark the object as deleted
	 */
	abstract protected function getDBKeyValueForDelete ($isRestoring=false);

	/**  Gets the key string for the associative array that contains field 
	 *	information for the table associated with this object.
     *	@see tables.inc.php
     *	@return string The key field string for the assocative array that 
     *		 contains field information for the table associated with this object.
     *	@access protected
	 */
	abstract protected function getTableConstant ();

	/**  
	 *	Checks if the data stored in this object is valid.	
     *	All this does is verify that there is valid data in the *required*
     *	fields.
     *	@param bool $checkKey If this is true, then the unique key field 
     *		 for the object is checked for validity as well
     *	@return bool True, if all required fields are valid.
	 */
	abstract public function requiredFieldsValid ($checkKey = false);

	/**  
	 *	Returns the unique ID for this object as 
	 *		 stored in memory (NOT THE DATABASE)
     *	@param bool $fieldName If true, then this function will return the 
     *		 table's field name for the unique ID instead of the actual unique ID
     *	@return mixed The unique ID field for this object.  If an array is 
     *		 returned, then more than one field makes up a key.
	*/
	abstract public function getUniqueID ($fieldName=false);

	/**
	 *	Caches this object to make queries more efficient.  Note that
	 *  this is not saved in the session.  But it will cut down dramatically
	 * 	on repeat querries in the same script
	 */
	public function cacheObject () 
	{
		$token = $this->getType().$this->getUniqueID();
		$GLOBALS['CACHE'][$token] = clone $this;
	}

	/**
	 * Removes all cached items from the cache global
	 */
	public function clearCache () 
	{
		$GLOBALS['CACHE'] = array ();
	}

	/**
	 * Removes only this item from the cache.
	 */
	public function removeFromCache () 
	{
        // Nexus cache
		if (isset ($GLOBALS['CACHE']))
		{
			unset($GLOBALS['CACHE'][$this->getType().$this->getUniqueID()]);                       
		}

        // ADODB cache        
        $sql = 'SELECT * FROM '.$GLOBALS[$this->getTableConstant()]['name'].' WHERE '.$this->getUniqueID (true).'='.$this->getUniqueID (false);        
        db()->CacheFlush($sql);
        
	}
	
	/** 
	 *	Retrieves the cached object, if any, that is of the same type
	 *	with the same ID.
	 *	@return mixed The iwDbTable-derived object that has been cached or
	 *			NULL if the object has not been cached.
	 */
	public function getCachedObject () 
	{
		$token = $this->getType().$this->getUniqueID();
		if (isset ($GLOBALS['CACHE'][$token])) 
		{			
			$obj = clone $GLOBALS['CACHE'][$token];						
			return $obj;
		}
		else 
		{
			return NULL;
		}
	}

	/**
	 * Loads data into the object in one call - you pass in the id 
	 * of the object - assumes that the id field is the 
	 */
	public function loadId($id)
	{
		if ($id != 0 && $id != '')
		{
			$this->id = $id;
			return $this->load();
		}
		else
		{
			
			return false;
		}
	}
	    
	/** 
	 *	Call this to load this object from the database.
	 * 	The unique ID property must be set before this function is called. The
	 * 	base class takes care of communicating with the database to populate
	 * 	the derived class's properties correctly.  	 	 
	 *	@return bool Returns true if loaded, false otherwise
	 */
	public function load () 
	{
		// Empty the error queue
		$this->clearErrors ();

		//echo "Class: ".get_class($this)."<br>";
		//echo "ID: ".$this->getUniqueID()."<br>";

		$object = $this->getCachedObject();
		if (!$object) 
		{
			if ($this->getUniqueID () != '') 
			{
				$query = 'SELECT * FROM '.$GLOBALS[$this->getTableConstant()]['name'].' WHERE '.$this->getUniqueID (true).'='.$this->getUniqueID (false);
                
                app()->flushCacheIfInvalid($query);
				$fields = app()->getDatabase()->CacheGetAssoc (DB_CACHE_EXPIRE,$query);

				if ($fields)	
				{
					$keyFieldName = str_replace ('\'', '', $this->getUniqueID (false));

					// Check to see if the record with the given ID was found.
					if (isset ($fields[$keyFieldName])) {

						// Add the key field to the array of fields.
						$fields[$keyFieldName][$this->getUniqueID (true)] = $keyFieldName;
						
						// Now load the data into the object.
						if ($this->setDBKeyValueArray ($fields[$keyFieldName])) 
						{
							//$this->cacheObject();
							return true;
						}
					}
				}
				else 
				{
					iwUtilities::logDatabaseError();
				}
			}
			else 
			{
				
			}
			return false;
		}
		else {
		    $this->iw_clone ($object);
			return true;
		}
	}

	/**  
	 *	Add this object to the database
     *	This assumes that the required properties have been filled in.
     *	@return bool True if the object was added successfully, false otherwise.
     */
	public function add () {

		
		// Empty the error queue
		$this->clearErrors ();

		if ($this->requiredFieldsValid ()) {

			$fields = $this->getDBKeyValueArray (false);

			// Insert into the database
			$table = $GLOBALS[$this->getTableConstant()]['name'];
			$result = app()->getDatabase()->AutoExecute ($table, $fields, 'INSERT');
			if ($result) 
			{			
				// The database query was a success
				if ($this->isNumericId())
				{
					$this->set (PROP_ID, mysql_insert_id());
				}

                app()->invalidateTable($table);
                
				// Broadcast a system notification
				$info = $this->getInfoForModRecord();				
				app()->notify (EVENT_OBJECT_ADDED,$info);

				return true;
			}

			iwUtilities::logDatabaseError();
		}
		return false;
	}

	/**
	 * Retrieves information in the form of an associative 
	 * array that can be used by the history tracking system
	 * when storing information about this object and some 
	 * action in the database.
	 *	@return array Returns an array with the following keys:
	 *					@arg @c string type The type of object this is 
	 *					@arg @c mixed id the unique Id for this record
	 *					@arg @c string username The username of the user who was logged in when the action was committed. 
	 */
	public function getInfoForModRecord ()
	{
		
		$id = $this->get(PROP_ID);
		$user = app()->getLoggedInUserObject();
		if ($user)
		{
			$username = $user->get(PROP_USERNAME);
			if ($this->getType() == OBJECT_TYPE_SHARE_USER)
			{
				$id = $username;
			}
		}
		else
		{
			if ($this->getType() == OBJECT_TYPE_SHARE_USER)
			{
				// If there's no logged in user and this
				//	modification is happening to a user 
				//	record then this is most likely a 
				//	login/logout action.  We can safely
				//	use the username of this object.
				$username = $this->get(PROP_USERNAME);				
			}
			else
			{
				$username = 'Guest';
			}
		}

		return array(
			'type'=>$this->getType(),
			'id'=>$id,
			'username'=>$username
		);
	}
	
	/**  
	 *	This will update a record in the database with the data stored in this object
     *	@return bool True if the object was updated successfully
	 */
	public function update () 
	{		
		// Empty the error queue
		$this->clearErrors ();

		if ($this->requiredFieldsValid (true)) 
		{			
			$this->removeFromCache();
			
			$fields = $this->getDBKeyValueArray (false);

			$where = $this->getUniqueID (true).'='.$this->getUniqueID ();

			// Update the database
			$table = $GLOBALS[$this->getTableConstant()]['name'];
			$result = app()->getDatabase()->AutoExecute ($table, $fields, 'UPDATE', $where);
						
			if ($result) 
			{
                app()->invalidateTable($table);
                
                // Make sure the latest db data is retrieved next time a load is done.
                $this->removeFromCache();
                
				// Broadcast a system notification
				app()->notify (EVENT_OBJECT_EDITED,$this->getInfoForModRecord());
				return true;
			}

			iwUtilities::logDatabaseError();
		}
		return false;
	}

	/** 
	 * This will mark the item as 'active' no matter what status it had previously
	 * 	@return bool Returns True if the object was restored successfully.
	 */
	public function restore()
	{
		
		// Empty the error queue
		$this->clearErrors ();

		// Get the where condition for this specific object.  It will
		//	be used in whatever SQL statement is used.
		$where = $this->getUniqueID (true).'='.$this->getUniqueID (false);

		$fields = $this->getDBKeyValueForDelete (true);
		if (is_array ($fields) && (count ($fields)>0))
		{
			// Update the database
			$table = $GLOBALS[$this->getTableConstant()]['name'];
			$result = app()->getDatabase()->AutoExecute ($table, $fields, 'UPDATE', $where);			
			if ($result) 
			{
				// The database query was a success
				return true;
			}

			iwUtilities::logDatabaseError();
			return false;
		}		
		
		$this->addError('This type of object ('.$this->getTypeName().') cannot be restored.', ERROR_TYPE_SERIOUS);
		return false;
	}
	
	/**  
	 * 	This will mark an object as deleted.
     *	Note that the object is not actually removed from the database.  Instead, it's
     *	simply marked as deleted.
     *	@return bool True if the object was marked as deleted successfully
	 */
	public function delete () 
	{		
		// Empty the error queue
		$this->clearErrors ();

		// Get the where condition for this specific object.  It will
		//	be used in whatever SQL statement is used.
		$where = $this->getUniqueID (true).'='.$this->getUniqueID (false);

		if ($this->getUniqueID () != '') 
		{
			$fields = $this->getDBKeyValueForDelete ();
			if (!$this->isTrueDeletionEnabled() && is_array ($fields) && (count ($fields)>0))
			{
				// Update the database
				$table = $GLOBALS[$this->getTableConstant()]['name'];
				$result = app()->getDatabase()->AutoExecute ($table, $fields, 'UPDATE', $where);							
				if ($result) 
				{
					// The database query was a success
					app()->notify (EVENT_OBJECT_TRASHED,$this->getInfoForModRecord());
					
					// The database query was a success
					return true;
				}

				iwUtilities::logDatabaseError();
			}
			else 
			{				
				// Delete the record completely
				$query = 'DELETE FROM '.$GLOBALS[$this->getTableConstant()]['name'].' WHERE '.$where;
				$result = app()->getDatabase()->Execute($query);
				if (!$result) 
				{
					iwUtilities::logDatabaseError();
				}
				else 
				{					
					$name = $this->hasProperty ('name') ? $this->get (PROP_NAME) : '';
					$type = $this->getTypeName (false);

					// The database query was a success
					app()->notify (EVENT_OBJECT_DELETED,$this->getInfoForModRecord());

					$this->onAfterPermanentDelete();
					
					return true;
				}
			}
		}
		else 
		{
			$this->addError (STR_211, ERROR_TYPE_SERIOUS);
		}
		return false;
	}

	/**  Makes sure that the given field values are unique to the table
     *	This will check the associated table for any rows that contains the
     *	same value for the same field as the ones given.
     *	@param string $field The name of the field to check
     *	@pararm mixed $value The value of the field to check.  IMPORTANT: If this is a string then it must be quoted before passing in.
     *	@return bool true if the field is unique, false if there is a duplicate.
	 */
	public function verifyFieldUnique ($field, $value) 
	{
		$query = 'SELECT * FROM '.$GLOBALS[$this->getTableConstant()]['name'].' WHERE '.$field.'='.$value;
        app()->flushCacheIfInvalid($query);
		$fields = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($fields !== false)	
		{
			if (count ($fields) == 0) 
			{
				return true;
			}
		}
		else 
		{
			iwUtilities::logDatabaseError();
		}

		return false;
	}

	/**
	 * Indicates whether or not the records are stored with unique numeric IDs
	 *	This causes the database handling code to automatically set the PROP_ID
	 * property to whatever the unique auto_numbering field returns upon 
	 * insertion. 
	 */
	public function isNumericId()
	{
		return true;
	}
		
	/**
	 * True deletion forces a complete removal of a record when 'delete' is called
	 *  Use this function to enable it or disable it.
	 *  @param bool $enable True to force the deletion, false to use the derived class's interpretation of delete
	 */
	public function enableTrueDeletion ($enable) 
	{
		$this->_trueDeletionEnabled = $enable;
	}

	/**
	 * Determines if true deletion is enabled.
	 *  @return bool True if enable,  false otherwise
	 */
	public function isTrueDeletionEnabled ()
	{
		return $this->_trueDeletionEnabled;
	}

	/**
	 * Retrieves the given unix timestamp in a format that can be stored
	 * in any iwDbTable field for any type of record.
	 * @return string The time as a database time string
	 */
	static public function getDatabaseTimeString ($time) 
	{
		return date ("Y-m-d H:i:s", $time);
	}
	
	/**
	 * Retrieves the icon representation for an instance of this item
	 * @param bool $large True if the larger icon is to be retrieved.
	 * @return string Returns the relative path (from the root of the site)
	 *					to the icon to use.
	 */
	abstract public function getItemIcon ($large=true);
	
	/** 
	 * Generates the HTML necessary to display this item in the
	 * 	browser.  The paramaters can be used for various types of 
	 *  to further specify the characteristics to be displayed to the
	 *	user.  
	 *	@param iwParameterCollection $params The collection of parameters that 
	 *			can alter the appearance of the output html.
	 *	@return string The HTML to display the item
	 *	@note This will display the item in a full page format.  Although
	 *			it doesn't require any specific width, the user should
	 *			not assume a maximum height of any kind.
	 */
	abstract public function render ($params=false);
	
	/**
	 * Asks that the object return the item in a format that will display nicely
	 * 	in a list of items of the same *or different* types. 
	 *	@param bool $selectable True if the item should have a checkbox (or
	 *					any other method of marking the item as selected)
	 *  @param bool $details True if the item should display details as well
	 *				as basic information or false to display just basic info.
     *  @param bool $actions Set to true to include action links, false otherwise.
	 *	@return string Returns a string containing the html to display the item.
	 */
	abstract public function renderAsListItem ($selectable, $details, $actions=true);
	
}
?>
