<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

define('FORM_ACTION_CANCEL','cancel');

/**
 * The base class for all forms in the applications.  Provides
 * field assembly, rendering, data storage and more.
 *
 * How to make a form:
 * Define a url action and return it in getFormAction (e.g. URLACTION_MODDOC)
 * Define a form type and return it in getFormType (e.g. FORMTYPE_CATEGORY)
 * Override getTitle and return the name of the form. Use 'getFormOrUrlId' 
 * 		to determine if user is editing or adding an object
 * Override loadData and call setData with the object being added,edited. If
 * 		adding a new object, you can create a new instance and call setData with
 * 		that instance.  If editing an object, get the ID from the url using getUrlId
 * 		and load the data from the database into an instance of the object. Use
 * 		setData to store the object in the form.  In both cases, first use recallFormData
 * 		to check if there's a cached version of the data that was stored upon 
 * 		submission of the form (but the form submission failed due to an input error,
 * 		for example).  If getFields (see below) is implemented right, this will 
 * 		redisplay the data from the previous submission so user doesn't have to 
 * 		reenter.
 * Override getFields and return an array of shFormField(or iwFormField) objects
 * 		that will be used to assemble the form.
 * Override _submit to take the submitted data and apply it how you see fit
 * Override getRedirectLink to indicate to the base where the user should be
 * 		redirected upon success.
 * 
 * Flexibility:
 * 1- To create the content of the form yourself, override getContentArea
 * 		and return the html for the *inside* of the form (no <form> tags).
 * 2- To display result data after submission, override getRedirectLink and
 * 		return an empty string AND override getResultsHtml and return the 
 * 		content you want to display
 * 
 * Example:
 * 	- Common Usage: shFormCategory, shFormGroup
 * 	- Flexible Usage: shFormDocument, shFormUsageReport 
 */
abstract class iwForm extends iwObject
{
	/**
	 * Used to make the form name unique. Generated
	 * during construction.
	 * @var string
	 */
	protected $_guid;
	
	/**
	 * Stores whatever information the derived
	 * form needs to store *after* submit takes place.
	 * @var mixed
	 */
	protected $_results = null;
	
	/**
	 * This (usually) stores an instance of the 
	 * object that this form represents
	 * @var iwObject
	 */
	protected $_data = null;

	/**
	 * This is the name chosen by the caller. If empty
	 * then a unique name will be generated.
	 * @var string
	 */
	protected $_name = '';
	
	/**
	 * If true, then this form is meant is being 
	 * used in an asyncronous way(AJAX).  
	 * @var bool 
	 */
	protected $_async = false;
	
	/**
	 * Constructor initializes the guid.
	 *
	 * @return iwForm
	 */
	public function iwForm()
	{
		$this->_guid = uniqid();	
	}
	
	/**
	 * Must override and return one of the FORMTYPE_X defines
	 */
	abstract public function getFormType();

	/**
	 * Must override and return one of the URLACTION_X defines
	 */
	abstract public function getFormAction();

	/**
	 * Override to add custom data to the template object
	 */
	public function addTemplateData(&$smarty){}
	
	/**
	 * Gets the results value as set by the form submission
	 * @return mixed The resuts
	 */
	public function getResults(){return $this->_results;}
	
	/**
	 * This function is called by the base if no redirect link is
	 * provided and the submit was successful.  The derived class should 
	 * return information about the submission
	 */
	public function getResultsHtml(){return '';}
	
	/**
	 * Gets the data associated with this form (usually an iwObject based
	 * object).
	 * @return mixed The stored data
	 */
	public function getData()
	{
		return $this->_data;
	}

	/**
	 * Sets the data associated with this form
	 * @param mixed $val The value of the data
	 */
	public function setData($val)
	{
		$this->_data = $val;
	}

	/**
	 * When set to true, the form will not display 
	 * a submit button when rendering and will use 
	 * a special post method that goes through XmlHttpRequests
	 * instead of the main page.
	 * @param bool $async True to use async, false otherwise.
	 */
	public function setAsync($async)
	{
		$this->_async = $async;	
	}

	/**
	 * Returns true if set to async, false otherwise.
	 * @return bool See description
	 */
	public function getAsync()
	{
		return $this->_async;	
	}
	
	/**
	 * Override this to add content after the form.  This 
	 * is useful for things like special javascript that 
	 * pertains to the form must be rendered outside of 
	 * the form.
	 *
	 * @return string The content or an empty string
	 */
	protected function getFormSuffixContent()
	{
		return '';
	}
	
	/**
	 * By default, this will use the fields returned
	 * by getFields to render html for each field and 
	 * concatenate them to create the content.
	 * @return string The HTML for the content area of the form
	 */
	protected function getContentArea(&$smarty)
	{                    
		$fields = $this->getFields();
        
		$formTypeField = new iwFormField(FIELD_TYPE_HIDDEN,$this->getFormType());
		$formTypeField->fieldName = FORM_TYPE;
		$fields[] = $formTypeField;

		if (!$this->getAsync())
		{
			$submitField = new iwFormField(FIELD_TYPE_SUBMIT,str('STR_SUBMIT'));
			$submitField->fieldName = 'submit';
			$fields[] = $submitField;
		}

		$formData = array();

		$content = '';    
        $index = 0;    
		foreach($fields as $field)
		{
            $field->setParentForm($this);
			if ($field->fieldHelp != '')
			{
				$content .= '<div class="help">'.$field->fieldHelp.'</div>';
			}        
                              
			$content .= $field->render();                           
		}
		                  
		return $content;
	}
	
	/**
	 * Adds the given set of parameters to the GET 
	 * parameters.  This is used by the async form display
	 * to pass in data that would normally be specified in the
	 * URL.
	 *
	 * @param unknown_type $arrayOfParameters
	 */
	public function addParams($arrayOfParameters)
	{
		foreach($arrayOfParameters as $newParam=>$newValue)
		{
			$_GET[$newParam] = $newValue;	
		}
	}
	
	/**
	 * Sets the hardcoded name for the form.  If none is 
	 * set then a default unique name will be used.
	 *
	 * @param string $name The name of the form
	 */
	public function setFormName($name)
	{
		$this->_name = $name;	
	}

    /**
     * By default the form id is the same as the form name
     *
     * @return string Returns a unique form id.
     */
    public function getFormId()
    {
        return $this->getFormName();
    }
    	
	/**
	 * By default the form name is the class name 
	 * plus a unique identifier.
	 *
	 * @return string Returns a unique form name.
	 */
	public function getFormName()
	{
		if (!$this->_name)
		{
			if ($this->_guid == '')
			{
				$this->_guid = uniqid();			
			}
			
			return get_class($this).$this->_guid;
		}
		else
		{
			return $this->_name;			
		}
	}
	
	/**
	 * Renders the form as html and returns it as a string
	 * @return string The HTML for the form
	 */
	public function render()
	{
		// First try to load the data
		if ($this->loadData())
		{
			$smarty = app()->getSmartyObject();
						
			$template = $this->getFormTemplate();
					
			$formData = array();
			$formData['async'] = $this->getAsync();
			$formData['title'] = $this->getTitle();
			$formData['subtitle'] = $this->getSubtitle();
			$formData['action'] = $this->getFormAction();
			$formData['method'] = $this->getFormMethod();
			$formData['default'] = $this->getDefaultField();
			$formData['edit'] = $this->isEditingObject();
			$formData['name'] = $this->getFormName();
			$formData['id'] = $this->getFormName();            
			$formData['successCallback'] = g('successcallback');
			            
			if ($template != '')
			{               
				$formData['content'] = $this->getContentArea($smarty);					            
                
				$smarty->assign('form',$formData);
				                
				$this->addTemplateData($smarty);                
				                
				$html = smarty_templ_fetch($template,$smarty);
				$html .= $this->getFormSuffixContent();                
				return $html;
			}
			else
			{                
				$smarty->assign('form',$formData);                
				$html = $this->getContentArea($smarty);                                
				$html .= $this->getFormSuffixContent();                
				return $html;
			}
		}
		else
		{
			$this->addError(str('STR_DATA_NOT_FOUND'));
		}
	}

	/**
	 * Returns the link to the object that is represented by this
	 * form.
	 * @return string A link to the view page for the object associated
	 * with this form or an empty string if no link could be generated.
	 */
	public function getRedirectLink() {return '';}

	/**
	 *  Returns the title of the form
	 *  @param string The title of the form
	 */	
	public function getTitle(){return '';}

	/**
	 * Returns the subtitle for the form
	 * @return string The subtitle of the form
	 */
	public function getSubtitle(){ return ''; }

	/**
	 * Gets the method that appears in the form tag (post/get)
	 * @return string The submit method used (post/get)
	 */
	public function getFormMethod()
	{
		return "post";
	}

	/**
	 * Override to handle submission for the form 
	 */
	protected function _submit() {return false;}


	/**
	 * Default behaviour is that form will be redisplayed
	 * if there was an error during processing. Override to
	 * change that behaviour.
	 */
	public function redisplayOnError ()
	{
		return true;
	}

	/**
	 * Gets the list of fields that are in the form
	 * @return array The array of iwFormField objects in the form
	 */
	public function getFields()
	{
		return array();
	}

	/**
	 * Usually this is not called because the form forces
	 * redirection and there's no chance for the form to be redisplayed.
	 * But there are some forms that display something specific as a result
	 * of Submission (find, for example) so no redirection occurs.
	 */
	public function redisplayOnSuccess ()
	{
		return false;
	}

	public function forgetFormData ()
	{
		$typeUniqueName = get_class ($this);
		$sessName = 'CACHE_FORM_DATA_'.$typeUniqueName;
		unset ($_SESSION[$sessName]);
	}

	/**
	 * Returns the field name of the field that should be given focus
	 * when the form is finished rendering.
	 * @return string The name of the field.
	 */
	public function getDefaultField (){return '';}

	/**
	 * Retrieves the unique action string associated with
	 * this form.  You can use this to identify the form.
	 * @return string The action string this object was initialized wi
	 */
	public function getAction ()
	{
		return 'index.php';
	}

	/**
	 * Determines whether or not the form is being cancelled (looks at 
	 * post variable indicating that cancel has bee pressed
	 * @return bool Returns true if the form is being cancelled false otherwise
	 */
	public function isCancellingForm ()
	{
		if (ps(FORM_ACTION_CANCEL) == FORM_CANCEL)
		{
			return true;
		}

		return false;
	}

	/**
	 * Determines if the form is being submitted by looking at post variable
	 * @return bool Returns true if the form is being submitted.
	 */
	public function isSubmittingForm()
	{
		if (ps(FORM_TYPE) == $this->getFormType())
		{
			return true;
		}

		return false;
	}
		
	/**
	 * Override to do authorization on this form.
	 * @return bool Returns true if authorization is granted
	 */
	public function authorize ()
	{
		return false;
	}

	/**
	 * Derived classes can override to supply their own template. By default, though,
	 * we display the fields as rendered by the form field templates.
	 * @return string The path to the form template (relative to the theme dir)
	 */
	protected function getFormTemplate()
	{
		return 'forms/form.tpl.html';
	}	


	/**
	 * Determines if the form should be displayed (based on the
	 * contents of the url parameter set)
	 * @param bool Returns true if the form should be displayed.
	 */
	public function isDisplayingForm ()
	{
		return (g(URL_ACTION_PARAM) == $this->getFormAction());
	}

	/**
	 * Submits the form. If the form object supplies 
	 * a view link then the form will redirect.
	 * @return bool Returns true if the form submission was successful.   
	 */
	public function submitForm()
	{			
		if ($this->_submit())
		{
			$link = $this->getRedirectLink();

            nx_fire_event('EVENT_FORM_REDIRECT_NEEDED',array('form'=>&$this,'link'=>&$link));
            
			if ($link)
			{				
				app()->redirect($link);
				exit();
			}
			else
			{
				$html = $this->getResultsHtml();
				if ($html != '')
				{
					app()->getActivePage()->addContent($html);
				}
			}

			return true;
		}

		return false;
	}

	/**
	 * This version of the form submit will only do the submission and
	 * not do any redirection as submitForm would.
	 * @return bool Returns true if the submit was succesful, false otherwise.
	 */
	public function submitFormAsync()
	{
		return $this->_submit();	
	}
	
	/**
	 * Gets the ID specified in the url making sure that it's a number
	 * before returning anything.
	 * @return bool Returns true if the form is being edited
	 */
	public function getUrlIdField ()
	{
		$id = g(URLARG_ID);
		if ($id != '')
		{
			return $id;
		}
		
		// There is no id associated with this object.
		return '';
	}
	
	/**
	 * Stores the data in the _data field to the cache overwriting any data
	 * already in the cache for forms of this type. 
	 */
	public function rememberFormData ()
	{
		$typeUniqueName = get_class ($this);
		$sessName = 'CACHE_FORM_DATA_'.$typeUniqueName;
		$_SESSION[$sessName] = $this->_data;
	}

	/**
	 * Retrieves the data stored in the session cache and applies it to
	 * the data field of this object.
	 * @return bool Returns true if the data was in the cache, false otherwise.
	 */
	public function recallFormData ()
	{
		$typeUniqueName = get_class ($this);
		$sessName = 'CACHE_FORM_DATA_'.$typeUniqueName;
		if (isset ($_SESSION[$sessName]))
		{
			$this->_data = $_SESSION[$sessName];
			return true;
		}

		return false;
	}
	
	/**
	 * Derived class overrides to verify that the
	 * data associated with the form can be loaded(from
	 * the database or whatever).  For example, if this 
	 * is an edit form, then the derived class may get
	 * the id of the object being edited from the url and 
	 * try to load it.  If it fails to load it then this
	 * function should return false.  The loaded data is
	 * stored in the _data field of this object using 
	 * setData.
	 * @return bool Returns true if data was loaded, false
	 * 				otherwise.
	 */
	protected function loadData()
	{
		return true;		
	}

	/**
	 * Determines if this form is editing existing 
	 * data.  Does by checking to see if there is 
	 * an 'id' field in the submitted data that is 
	 * non-empty.
	 *
	 * @return unknown
	 */
	function isEditingObject()
	{
		$id = $this->getFormOrUrlId();
		if ($id != '' && $id != '0')
		{			
			return true;
		}		
		
		return false;
	}
	
	/**
	 * Looks at both the url and the post data to determine
	 * what ID is associated with the form.
	 * @return string The ID found or an empty string if no id found. 
	 */
	public function getFormOrUrlId()
	{
		$id = ps(PROP_ID);		
		if ($id != '' || (is_numeric($id) && $id != 0))
		{			
			return $id;
		}		

		$id = g(URLARG_ID);
		if ($id != '' || (is_numeric($id) && $id != 0))
		{
			return $id;
		}			
		
		return '';
	}
		
}

?>
