<?php
/*
 * Nexus is a web-based file management application.
 * Copyright (C) 2007 Karim Shehadeh
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

define ('DOMAIN_NONE',0);
define ('DOMAIN_GLOBAL',1);
define ('DOMAIN_COMPANY',2);
define ('DOMAIN_GROUP',3);

/**
 * This is used to represent a group and company as a single
 *	pair.  This is useful when representing entities which have 
 *	access to an item.
 */
class shCompanyGroupPair extends iwObject
{
	/**
	 * The group object (shGroup)
	 */
	protected $_group = null;
	
	/**
	 * The company object (shCompany)
	 */
	protected $_company = null;	
	
	/**
	 * Verifies that the given company and group actually are joined as a domain in the
	 * database.
	 * @return bool Returns true if it is a domain, false otherwise.
	 * @param $companyId number The company id
	 * @param $groupId number The group id
	 */
	static public function checkDomainExists($companyId,$groupId)
	{
		$count = app()->getDatabase()->GetOne('select count(*) from sh_company_group_map where company_id='.$companyId.' and group_id='.$groupId);
		if ($count > 0)
		{
			return true;
		}		
		return false;
	}
	
	/**
	 * Returns all the domains that are accessible given the domain restriction passed in
	 * to the function.  For example, DOMAIN_GLOBAL has access to all domains and so all domains
	 * will be returned.
	 * @return array An array of shCompanyGroupPair objects
	 * @param $includeEveryoneChoice bool[optional] True if the 'Everyone' or 'Public' option should be included
	 * @param $domainRestriction int[optional] Which domains should be allowed to be returned.
	 */
	static public function getAccessibleDomains($includeEveryoneChoice=false, $domainRestriction=DOMAIN_GLOBAL)
	{        
		$companies = new shCompanyCollection;
        
		$user = app()->getLoggedInUserObject();
		if (!$user) 
        {
            return array();
        }
		
		$companyList = array();
		$singleGroup = null;
		if ($domainRestriction == DOMAIN_GLOBAL)
		{            
			$params = new iwParameterCollection();			
            $params->add(new iwParameter(FILTER_SORTBY,'name'));
            $params->add(new iwParameter(FILTER_SORTDIR,'asc'));
            
			$params->add(new iwParameter (FILTER_INCLUDE_AGGREGATE,true));
			$companies->load($params);
			$companyList = $companies->items();			
		}
		else if ($domainRestriction == DOMAIN_COMPANY)
		{
			$companyList[] = $user->getCompany();
		}
		else if ($domainRestriction == DOMAIN_GROUP)
		{
			$companyList[] = $user->getCompany();
			$singleGroup = $user->getGroup();
		}   
		        		
		foreach ($companyList as $company)
		{
			$currentCompany = $company->get(PROP_NAME);
			$currentCompanyId = $company->get(PROP_ID);
			
			$groups = array();
			if (!$singleGroup)
			{
				$groups = $company->getGroups();
				
				if ($includeEveryoneChoice && ($domainRestriction != DOMAIN_GROUP))
				{
					$groups[] = shGroup::getAllGroup();			
				}
			}
			else
			{
				$groups[] = $singleGroup;
			}
			
			foreach ($groups as $group)
			{				
				$domain = new shCompanyGroupPair();
				$domain->setCompany($company);
				$domain->setGroup($group);
				$domains[] = $domain;				
			}			
		}		
		return $domains;		
	}
	
	/**
	 * Loads the domain from the database automatically.  Also verifies that
	 * the company/group combo actually exists.
	 * @return bool Returns true if loaded successfully
	 * @param $companyId number The id of the company
	 * @param $groupId number The id of the group.
	 */
	public function load ($companyId, $groupId)
	{
		if ($this->checkDomainExists($companyId,$groupId))
		{
			$group = new shGroup;
			if ($group->loadId($groupId))
			{
				$this->setGroup($group);	
			}
			
			$company = new shCompany;
			if ($company->loadId($companyId))
			{
				$this->setCompany($company);
			}		
			
			return true;				
		}	
		
		return false;
	}
	
	/**
	 * This will ensure that *all* information regarding the contained company and group
	 * are loaded from the database.  If there is already detailed information, then nothing
	 * will be done.
	 */
	public function loadDetailedInfo()
	{
		if ($this->_group && $this->_group->get(PROP_NAME) == '')
		{
			$this->_group->load();
		}	
		
		if ($this->_company && $this->_company->get(PROP_NAME) == '')
		{
			$this->_company->load();
		}	
	}
	
	/**
	 * Sets the group object for this pair
	 *	@param shGroup $groupObject The group object
	 */
	function setGroup ($groupObject)
	{
		$this->_group = $groupObject;
	}
	
	/**
	 * Sets the company for this pair
	 * 	@param shCompany $companyObject The company object.
	 */
	function setCompany ($companyObject)
	{
		$this->_company = $companyObject;
	}
	
	/**
	 * Retrieves a reference to the group object in this 
	 *	company/group pair
	 *	@return shGroup The group object.
	 */
	function &getGroup ()
	{
        if ($this->_group == null)
        {
		    $this->setGroup(new shGroup);
        }
        
        return $this->_group;
	}
	
	/**
	 * Retrieves a reference to the company object in this 
	 *	company/group pair
	 *	@return shCompany The company object.
	 */
	function &getCompany ()
	{
        if ($this->_company == null)
        {
		    $this->setCompany(new shCompany);
        }
        
        return $this->_company;
	}
	
	/**
	 * Retrieves all users who are a member of this domain
	 * @return array An array of shUser objects
	 */
	function getUsers()
	{
		$users = array();
				
		$query =
			'select iw_users.* from iw_users, sh_user_access_map where
			sh_user_access_map.group_id = '.$this->_group->get(PROP_ID).
			' and sh_user_access_map.company_id = '.$this->_company->get(PROP_ID).
			' and iw_users.username = sh_user_access_map.username';
			
        app()->flushCacheIfInvalid($query);
		$res = app()->getDatabase()->CacheGetAll(DB_CACHE_EXPIRE,$query);
		if ($res !== false)
		{
			foreach ($res as $result)
			{
				$user = new shUser;
				$user->setDBKeyValueArray($result);
				$users[] = $user;
			}
		}
		else
		{
			app()->addErrorObject ($res2,ERROR_TYPE_SERIOUS);
		}
			
		return $users;		
	}
	
	/**
    * @desc Returns a hierarchical list of domains and groups ordered like this
    *       Domain A
    *           Group A
    *           Group B
    *       Domain B
    *           Group C
    *           Group D
    *           ...
    *       ...
    *   The associative array looks like this:
    *       [domains]=>
    *               name
    *               id
    *               selected
    *               [groups] =>
    *                   name
    *                   id
    *                   selected
    * 
    *   @param mixed $obj The object from which currently selected items will be retrieved.
    *   @param bool $allowAggregates True to show the 'aggregate' options such as "Everyone"
    */
    static public function getHierarchicalDomainList($obj,$allowAggregates)
    {
        $permission = app()->getLoggedInUserObject()->getPermissions();
            
        $domainRestriction = DOMAIN_NONE;
        $allowEveryone = false;
        $permissionGroup = $permission->getPermissionGroup();
        
        if ($permissionGroup == USER_TYPE_ADMIN)
        {
            $domainRestriction = DOMAIN_GLOBAL;
            $allowEveryone = true;
        }
        else if ($permissionGroup == USER_TYPE_DOMAIN_ADMIN)
        {            
            $domainRestriction = DOMAIN_COMPANY;
        }
        else if ($permissionGroup == USER_TYPE_DOMAIN_GROUP_ADMIN)
        {
            $domainRestriction = DOMAIN_GROUP;
        }        
                                
        $companies = new shCompanyCollection;

        $user = app()->getLoggedInUserObject();
        if (!$user) 
        {
            return array();
        }
        
        $selectedDomains = $obj->getDomains();
        
        $companyList = array();
        $singleGroup = null;
        if ($domainRestriction == DOMAIN_GLOBAL)
        {
            $params = new iwParameterCollection();            
            $params->add(new iwParameter (FILTER_INCLUDE_AGGREGATE,true));
            $params->add(new iwParameter(FILTER_SORTBY,'name'));
            $params->add(new iwParameter(FILTER_SORTDIR,'asc'));
            $companies->load($params);
            $companyList = $companies->items();            
        }
        else if ($domainRestriction == DOMAIN_COMPANY)
        {
            $companyList[] = $user->getCompany();
        }
        else if ($domainRestriction == DOMAIN_GROUP)
        {
            $companyList[] = $user->getCompany();
            $singleGroup = $user->getGroup();
        }                          

        $domains = array();
        foreach ($companyList as $company)
        {
            $companyArray = $company->toArray();
            $companyArray['selected'] = false;

            foreach($selectedDomains as $domain)
            {
                
                if (($domain->getCompany()->get(PROP_ID) == $company->get(PROP_ID)) &&
                    ($domain->getGroup()->get(PROP_ID) == GROUP_ALL))
                {
                    $companyArray['selected'] = true;
                    break;
                }
            }
            
            $companyArray['groups'] = array();
            $currentCompanyGroups = $company->getGroups();
            foreach($currentCompanyGroups as $group)
            {
                $groupArray = $group->toArray();
				$groupArray['selected'] = false;
                
                foreach($selectedDomains as $domain)
                {
                    if (($domain->getCompany()->get(PROP_ID) == $company->get(PROP_ID)) &&
                        ($domain->getGroup()->get(PROP_ID) == $group->get(PROP_ID)))
                    {
                        $groupArray['selected'] = true;
                        break;
                    }
                }
                
                $companyArray['groups'][] = $groupArray;
            }            
            
            $domains[] = $companyArray;            
        }        
    
        return $domains;
    }
	
}

?>