<?php

class shDocumentUploadServer
{
	protected $_inputFiles = array();
	protected $_outputFiles = array();
	protected $_purgatoryFiles = array();
    protected $_recentFiles = array();

	/**
	 * The constructor will initialize the purgatory files array
	 */
	public function __construct()
	{
		// Retrieve the purgatory files from the session.
		$this->_purgatoryFiles = app()->retrieveValue(PURGATORY_FILES);
		if (!is_array($this->_purgatoryFiles))
		{
			$this->_purgatoryFiles = array();
		}               

        $this->_recentFiles = app()->retrieveValue(RECENT_FILES);
        if (!is_array($this->_recentFiles))
        {
            $this->_recentFiles = array();
        }               
    }
	
	/**
	 * The destructor write out the purgatory files to the SESSION
	 */
	public function __destruct()
	{
		// Write out purgatory files
		app()->rememberValue(PURGATORY_FILES,$this->_purgatoryFiles);
        
        // Write out recent files
	    app()->rememberValue(RECENT_FILES,$this->_recentFiles); 
    }
	
	/**
	 * Adds this to the array of files that will be uploaded when doUpload is called
	 *	@param shUploadDocumentProperties $fileProperties The file properties of the file to upload
	 * 	@return Returns true if added to the list of files to "upload"
	 */
	public function addFile(shUploadDocumentProperties $fileProperties)
	{
		$this->_inputFiles[] = $fileProperties;
		return true;
	}
	
    /**
    * @desc  Retrieves an array of all purgatory files
    * @return array An array of shUploadDocumentProperties objects.
    */
    public function getPurgatoryFiles()
    {
        return $this->_purgatoryFiles;
    }
    
    
    /**
    * @desc  Retrieves an array of all recently uploaded files
    * @return array An array of shUploadDocumentProperties objects.
    */
    public function getRecentFiles()
    {
        return $this->_recentFiles;
    }    
    
    /**
    * @desc Removes all recent files from memory
    */
    public function clearUserRecentFiles()
    {
        $this->_recentFiles = array();
    }
    
	/**
	 * Clears the upload purgatory of all files 
	 */
	public function clearUserPurgatory()
	{        
		foreach ($this->_purgatoryFiles as $file)
		{
			$destinationPath = $this->getPurgatoryFolder();
			if ($destinationPath)
			{                
				$destinationPath .= '/'.$this->getPurgatoryFile($file->guid,$file->originalFilename);
				if (file_exists($destinationPath))
				{
					@unlink($destinationPath);
				}
			}			
		}
		
		// Empty the array
		$this->_purgatoryFiles = array();
	}
	
	/**
	 * Performs the actual upload for all files in the queue
	 * @param int $destination The destination for the upload (library or filebox)
	 * @return Returns the number of files that were uploaded successfully.
	 */
	public function doUpload()
	{
		$this->_outputFiles = array();
		
		foreach ($this->_inputFiles as $file)
		{            
			if ($file->uploadLocation == DEST_UPLOAD_LIBRARY)
			{
				$this->uploadToLibrary($file);
			}
			else if ($file->uploadLocation == DEST_UPLOAD_FILEBOX)
			{
				$this->uploadToFileBox($file);
			}
			else if ($file->uploadLocation = DEST_UPLOAD_PURGATORY)
			{
				$this->uploadToPurgatory($file);
			}
		}
		
		$this->_inputFiles = array();
	}
	
	/**
	 * Retrieves the array of input file properties that will be processed
	 * 	when doUpload is called
	 * @return array The array of shUploadDocumentProperties objects
	 */
	public function getInputFiles()
	{
		return $this->_inputFiles;
	}

	/**
	 * Retrieves the array of output file properties that will be populated
	 * 	when doUpload has completed.
	 * @return array The array of shUploadDocumentProperties objects
	 */
	public function getOutputFiles()
	{
		return $this->_outputFiles;
	}
	
	/**
	 * Uploads the given file to the Filebox by adding it to the database and moving
	 * the physical file to the proper location
	 * @param shUploadDocumentProperties $file The properties of the file to upload
	 */
	protected function uploadToFileBox(shUploadDocumentProperties $file)
	{
	    $successFiles = array();
	    $errorFiles = array();

	    $user = app()->getLoggedInUserObject();
	    $permissions = $user->getPermissions()->getPermissionGroup();

        $item = new shPoolItem;    

        $originalFileName = $file->originalFilename;
        $currentFilePath = $file->localFilePath;

        $maxSizeInMb = $user->get(PROP_ITEMPOOL_SIZE);

        // Check for room on disk for this document (if max is 0 that means
        //    filebox is unlimited
        if ($maxSizeInMb > 0)
        {
            $size = filesize($file->localFilePath);                
            if (!shFileAnalysis::isFileBoxSpaceAvailable($size, $maxSizeInMb))
            {
                $file->uploadResult = UPLOAD_RES_FAIL;
 				$file->uploadResultMessage = str('STR_FILEBOX_NOSPACE');
            }                
            else
            {
                $name = iwUtilities::basenameNoExtension($originalFileName);

                $type = typeman()->determineNexusType($originalFileName,'');        
                $item->loadDocument($type,$originalFileName,$currentFilePath);
                $item->set(PROP_NAME,$file->name != '' ? $file->name : $name);
                $item->set(PROP_DESCRIPTION,$file->description);
                $item->set(PROP_STATUS,ITEM_STATUS_ACTIVE);
                $item->set(PROP_USERNAME,$user->get(PROP_USERNAME));
								
                if ($item->add ())
                {            
                    // This will rename the file and delete the original
                    $file->fileSize = filesize($currentFilePath);
                    $file->databaseRecordId = $item->get(PROP_ID);
                    $item->storeFileToFinalLocation ();            
                    $file->uploadResult = UPLOAD_RES_SUCCESS;
					$file->uploadResultMessage = str('STR_UPLOAD_FILE_FILEBOX');

                    nx_fire_event('EVENT_OBJECT_ADDED',array('object'=>$item));
                }       
                else
                {
					$file->uploadResult = UPLOAD_RES_FAIL;
                    $file->uploadResultMessage = str('STR_DOC_COPY_FAILED2');
                }     
            }
        }
        else
        {
			$file->uploadResult = UPLOAD_RES_FAIL;
            $file->uploadResultMessage = str('STR_FILEBOX_NOSPACE');
        }                              

        $this->_recentFiles[] = $file;
        $this->_outputFiles[] = $file;
	}
	
	protected function uploadToLibrary($file)
	{
		$user = app()->getLoggedInUserObject();
		$permissions = $user->getPermissions()->getPermissionGroup();
		$categoryId = $file->parentCategoryId;	
        l('Category: '.$categoryId);
        
		$item = new shItemDocument;

		$originalFileName = $file->originalFilename;
		$currentFilePath = $file->localFilePath;
		$item->set(PROP_NAME,$file->name ? $file->name : $originalFileName);
		$item->set(PROP_DESCRIPTION,$file->description);
		$item->set(PROP_STATUS,ITEM_STATUS_ACTIVE);

		if (count($file->domainsWithAccess) > 0)
		{
			foreach($file->domainsWithAccess as $domain)
			{
				if ($domain instanceof shCompanyGroupPair)
				{
					$item->addCompanyGroup($domain->getCompany()->get(PROP_ID),
										   $domain->getGroup()->get(PROP_ID));
				}
			}
		}
		else
		{
			// No domains were specified so pick some that are appropriate
			//	for the currently logged in user.
			if ($permissions == USER_TYPE_ADMIN)
			{			
				$item->addCompanyGroup(COMPANY_ALL,GROUP_ALL);
			}
			else if ($permissions == USER_TYPE_DOMAIN_ADMIN)
			{
				$item->addCompanyGroup($user->getCompany()->get(PROP_ID),GROUP_ALL);
			}
			else if ($permissions == USER_TYPE_DOMAIN_GROUP_ADMIN)
			{
				$item->addCompanyGroup($user->getCompany()->get(PROP_ID),
					$user->getGroup()->get(PROP_ID));
			}
		}

		$uploadsDir = app()->get('upload_dir');

		$name = $originalFileName;
		$name = str_replace(array('"',' '),'_',$name);	
		$tempPath = $uploadsDir.'/'.$name;

		$moved = false;

		if (shAuthorize::canEditCategory($user, $categoryId))
		{
			$item->addCategory($categoryId);

			$type = typeman()->determineNexusType($tempPath,'');		
			$item->loadDocument($type,$originalFileName,$currentFilePath);

			// Domains
			$selectedCG = ps('domains');
			if (!is_array($selectedCG))
			{
				if ($selectedCG != '')
				{
					$selectedCG = array ($selectedCG);
				}
				else
				{   
					$selectedCG = array (COMPANY_ALL.','.GROUP_ALL);
				}
			}

			// Replace the current set with what we have here.
			$item->clearCompanyGroupPairs();
			foreach ($selectedCG as $cg)
			{
				list($company,$group) = split(',',$cg);
				$item->addCompanyGroup ($company,$group);			
			}


			if ($item->add ())
			{			
				// This will rename the file and delete the original
                $file->fileSize = filesize($currentFilePath);
                $file->databaseRecordId = $item->get(PROP_ID);
				$item->storeFileToFinalLocation ();			                
				$file->uploadResult = UPLOAD_RES_SUCCESS;
				$file->uploadResultMessage = str('STR_UPLOAD_FILE_LIBRARY');
			}
			else
			{
				$file->uploadResult = UPLOAD_RES_FAIL;
				$file->uploadResultMessage = str('STR_DOC_COPY_FAILED2');
			}
		}
		else
		{
			$file->uploadResult = UPLOAD_RES_FAIL;
			$file->uploadResultMessage = str('STR_DOC_CANNOT_ADD_FILES_TO_CAT');
		}
		
        $this->_recentFiles[] = $file;
		$this->_outputFiles[] = $file;
	}		

	protected function getPurgatoryFolder()
	{
		// Store in the purgatory area of the cache
		$temporaryPath = app()->get(PROP_TEMP_DIR);
		$temporaryPath .= '/'.app()->getLoggedInUsername();
		
		$continue = false;
		if (!file_exists($temporaryPath))
		{
			if (!@mkdir($temporaryPath))
			{
				return false;
			}			
		}

		return $temporaryPath;
	}
	
	protected function getPurgatoryFile($id,$filename)
	{
		return $id.'_'.$filename;
	}
	
	protected function uploadToPurgatory($file)
	{
		// Store in the purgatory area of the cache        
		$temporaryPath = app()->get(PROP_TEMP_DIR);
		$temporaryPath .= '/'.app()->getLoggedInUsername();
		
		$destinationPath = $this->getPurgatoryFolder();
        $continue = true;
		if (!$destinationPath)
		{
			$file->uploadResult = UPLOAD_RES_FAIL;
			$file->uploadResultMessage = str('STR_UNABLE_TO_CREATE_DIR');
			$continue = false;
		}
		
		if ($continue)
		{
			$id = uniqid();
			$destinationPath .= '/'.$this->getPurgatoryFile($id,$file->originalFilename);
			
            if (is_uploaded_file($file->localFilePath))
            {
                $continue = @move_uploaded_file($file->localFilePath,$destinationPath);
            }
            else
            {
                $continue = @copy($file->localFilePath,$destinationPath);                
            }
            
			if ($continue)
			{
                $file->fileSize = filesize($destinationPath);
                $file->currentFilePath = $destinationPath;
                
				$file->guid = $id;
				$this->_purgatoryFiles[] = $file;
                $file->uploadResult = UPLOAD_RES_SUCCESS;
                $file->uploadResultsMessage = '';            
			}
		}
		
		$this->_outputFiles[] = $file;
        $this->_recentFiles[] = $file;
	}
}

?>