<?php

define ('DS_ERR_NONE',0);
define ('DS_ERR_TEMP_FILE_DELETE_FAILED',1);
define ('DS_ERR_LOCAL_COPY_FAILED',2);
define ('DS_ERR_DOC_NOT_FOUND',3);
define ('DS_ERR_EXISTING_DOC_DELETE_FAILED',4);
  
/**
* @desc The document storage classes are used as an interface to various document
*       storage mechanisms.  This class and its derivations act as a sort of librarian
*       wherein a client asks this object to retrieve a document from the "shelves" and 
*       can also ask that a new document be added.  
*       Use "retrieveDocument" to get an existing document 
*       Use "storeDocument" to add a new document or replace an existing one.
*       Use "deleteDocument" to delete an existing document
*       Use "removeTempDocument" to clear out a document in its temporary storage location
*           after it has been retrieved using "retrieveDocument
* 
*       When implementing a new storage mechanism, you must set the _storageLocationId to a
*       unique value.  In order to ensure uniqueness, please register your storage location id 
*       with iWonder Designs.  Send an email with details about your storage mechanism to 
*       karim@iwonderdesigns.com
* 
*/
abstract class shDocumentStorage
{                 
    protected $_storageLocationId = 0;
    protected $_temporaryRetrievalLocation='';

    public function getLastError() {return $this->_lastError;}
    protected function setLastError($err) {$this->_lastError = $err;}
    
    public function getStorageLocationId() {return $this->_storageLocationId;}
    
    public function getTemporaryRetrievalLocation() {return $this->_temporaryRetrievalLocation;}
    public function setTemporaryRetrievalLocation($loc) {$this->_temporaryRetrievalLocation = $loc;}
    
    /**
    * @desc If this returns true then 'retrieveDocument' returns the path to a local temporary
    *       location from which Nexus will serve the document to its users.  If it returns false,
    *       then the path returned is actually a URL served by a third-party.
    * @return bool Returns true if local storage is used or false if url links are used.
    */
    abstract public function isLocalStorageSupported();
    
    /**
    * @desc This will retrieve a document and store it in the temporary document cache location.
    *       Caller must call 'removeTempDocument' in order to remove it from the temporary location
    * @param shItemDocument $doc The document object to retrieve
    * @return string The full path to the temporary location of the file or null if failed.  Use
    *           getLastError to find out what happened.
    */
    abstract public function retrieveDocument(shItemDocument $doc);
    
    /**
    * @desc This will store the document in the final storage location.  Use
    *       retrieveDocument to get it.
    * @param shItemDocument $doc The document object to store
    * @return bool Returns false if store failed
    */
    abstract public function storeDocument(shItemDocument $doc);

    /**
    * @desc Permanently removes the document from the data store
    * @param shItemDocument $doc The document object to delete
    * @return bool Returns false if delete failed.
    */
    abstract public function deleteDocument(shItemDocument $doc);
    
    /**
    * @desc Retrieves the file size on disk
    * @param shItemDocuemnt $doc The document object
    * @return int The size of the file in bytes
    */
    abstract public function getFileSize(shItemDocument $doc);
    
    /**
    * @desc Retrieve detailed document information regarding things like
    *       filesize, creation date, modification date and other filesystem
    *       related attributed
    * @param shItemDocument $doc The document object
    * @return shFileInfo The file information object.
    */
    abstract public function getDocumentInfo(shItemDocument $doc);
    
    /**
    * @desc Removes the document from the temporary store after retrieveDocument has been
    *       called.
    * @param int $tempDocument The temporary document to remove
    * @return bool Returns false if temp delete failed.
    */
    public function removeTempDocument(shItemDocument $doc, $tempFilePath)
    {        
        if ($file_exists($tempDocument))
        {
            if (@unlink($tempDocument))
            {
                $this->setLastError(DS_ERR_NONE);
                return true;
            }
            else
            {
                $this->setLastError(DS_ERR_TEMP_FILE_DELETE_FAILED);
                return false;
            }
        }
        else
        {
            $this->setLastError(DS_ERR_TEMP_FILE_DELETE_FAILED);
            return false;
        }
    }
    
    /**
    * @desc Creates the temporary location into which the 
    *       the file is copied/reserved for temporary use.
    * @param int $id The id of the document
    * @return string The full path to the file or false if the path could
    *           not be generated.
    */
    protected function generateTemporaryFileLocation(shItemDocument $doc)
    {
        if ($this->_temporaryRetrievalLocation != '' &&
            file_exists($this->_temporaryRetrievalLocation))
        {
            if ($this->_temporaryRetrievalLocation[strlen($this->_temporaryRetrievalLocation)-1] != '/' &&
                $this->_temporaryRetrievalLocation[strlen($this->_temporaryRetrievalLocation)-1] != '\\')
            {
                $this->_temporaryRetrievalLocation .= '/';
            }
            
            $randomNumber = rand(500,5000);
            return $this->_temporaryRetrievalLocation.'_'.$randomNumber.'_'.$doc->get(PROP_ID).'_'.$this->_storageLocationId;
        }
        else
        {
            return false;
        }
    }
}

?>
